# -*- coding: utf-8 -*-
"""Post install import steps for collective.pamfixes."""

# python imports
import pkg_resources

# zope imports
from Products.CMFPlone.interfaces import INonInstallable
from plone import api
from zope.interface import implementer

# local imports
from collective.pamfixes import logger


TESTFIXTURE_ADD_ONS = [
    'archetypes.multilingual',
    'plone.app.contenttypes',
    'plone.multilingualbehavior',
]


@implementer(INonInstallable)
class HiddenProfiles(object):

    def getNonInstallableProfiles(self):
        """Hide uninstall profile from site-creation and quickinstaller"""
        return [
            'collective.pamfixes:testfixture',
        ]


def post_install(context):
    """Post install script"""
    # Do something at the end of the installation of this package.


def post_install_testfixture(context):
    """Post install script for testfixture environments."""
    if is_not_testfixture_profile(context):
        return

    setup_pam(context)
    setup_sample_content(context)


def uninstall(context):
    """Uninstall script"""
    # Do something at the end of the uninstallation of this package.


def is_not_testfixture_profile(context):
    return context.readDataFile(
        'collectivepamfixes_testfixture_marker.txt'
    ) is None


def setup_sample_content(context):
    """Setup sample content."""
    portal = api.portal.get()
    root_en = portal.get('en')

    sample_folder = root_en.get('sample-folder')
    if not sample_folder:
        sample_folder = api.content.create(
            container=root_en,
            type='Folder',
            id='sample-folder',
            title=u'Sample Folder',
            description=u'This folder was created for testing purposes',
        )


def setup_pam(context):
    """Setup a multilingual site using p.a.m."""
    try:
        from plone.app.multilingual.browser.setup import SetupMultilingualSite
    except ImportError:
        logger.info(
            'plone.app.multilingual is not available. '
            'The setup of a multilingual site is not possible.'
        )
        return

    # Install required add-ons, if available
    qi = api.portal.get_tool(name='portal_quickinstaller')
    for add_on in TESTFIXTURE_ADD_ONS:
        try:
            pkg_resources.get_distribution(add_on)
        except pkg_resources.DistributionNotFound:
            logger.info('Add-on {0} not available.'.format(add_on))
            continue
        if not qi.isProductInstalled(add_on):
            try:
                qi.installProduct(add_on)
            except Exception:
                logger.warning(
                    'Add-on {0} could not be installed.'.format(add_on)
                )
                continue
            else:
                logger.info(
                    'Add-on {0} successfully installed.'.format(add_on)
                )

    # Define available languages
    language_tool = api.portal.get_tool(name='portal_languages')
    languages = language_tool.getSupportedLanguages()
    for lang in ['en', 'es', 'de']:
        if lang not in languages:
            language_tool.addSupportedLanguage(lang)

    # Enable request negotiator
    language_tool.use_request_negotiation = True

    site = api.portal.get()
    setup_tool = SetupMultilingualSite()
    setup_tool.setupSite(site)
    try:
        SetupMultilingualSite(site).move_default_language_content()
    except AttributeError:
        logger.info('move_default_language_content not supported.')
    logger.info('Setup of multilingual site finished.')



# -*- coding: utf-8 -*-
"""collective.pamfixes custom viewlets."""

# python imports
import pkg_resources
from pkg_resources import parse_version

# Plone imports
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import IPloneSiteRoot
from plone.app.multilingual.browser.selector import getPostPath
from plone.app.multilingual.browser.viewlets import (
    AlternateLanguagesViewlet)
from plone.memoize import ram, view
from zope.component import getUtility

# local imports
from collective.pamfixes.interfaces import ITranslationManager

pam_distribution = pkg_resources.get_distribution('plone.app.multilingual')
pam_version = pam_distribution.version


def _cache_until_catalog_change_post_path(fun, self):
    catalog = getToolByName(self.context, 'portal_catalog')
    key = '{0}{1}{2}/{3}'
    key = key.format(
        fun.__name__,
        catalog.getCounter(),
        '/'.join(self.context.getPhysicalPath()),
        self.post_path,
    )
    return key


class CustomAlternateLanguagesViewlet(AlternateLanguagesViewlet):
    """Notify search engines about alternates languages of current content
    item.
    """

    @property
    @view.memoize
    def post_path(self):
        return getPostPath(self.context, self.request)

    @ram.cache(_cache_until_catalog_change_post_path)
    def get_alternate_languages_pam_1_x(self):
        """Cache relative urls only. If we have multilingual sites and multi
        domain site caching absolute urls will result in very inefficient
        caching. Build absolute url in template.
        For plone.app.multilingual 1.x
        """
        tm = ITranslationManager(self.context)
        catalog = getToolByName(self.context, 'portal_catalog')
        results = catalog(
            TranslationGroup=tm.query_canonical(),
            Language='all',
        )

        plone_site = getUtility(IPloneSiteRoot)
        portal_path = '/'.join(plone_site.getPhysicalPath())
        portal_path_len = len(portal_path)

        alternates = []
        for item in results:
            url = item.getURL(relative=1)
            path_len = len('{0:s}/'.format(item.Language))
            if url.startswith(portal_path):
                path_len += portal_path_len
            url = url[path_len:]
            if self.post_path:
                url = '/'.join([url.strip('/'), self.post_path.strip('/')])
            alternates.append({
                'lang': item.Language,
                'url': url.strip('/'),
            })

        return alternates

    @ram.cache(_cache_until_catalog_change_post_path)
    def get_alternate_languages_pam_2_x(self):
        """Cache relative urls only. If we have multilingual sites and multi
        domain site caching absolute urls will result in very inefficient
        caching. Build absolute url in template.
        For plone.app.multilingual 2.x
        """
        tm = ITranslationManager(self.context)
        catalog = getToolByName(self.context, 'portal_catalog')
        results = catalog(TranslationGroup=tm.query_canonical())

        plone_site = getUtility(IPloneSiteRoot)
        portal_path = '/'.join(plone_site.getPhysicalPath())
        portal_path_len = len(portal_path)

        alternates = []
        for item in results:
            url = item.getURL(relative=1)
            path_len = len('{0:s}/'.format(item.Language))
            if url.startswith(portal_path):
                path_len += portal_path_len
            url = url[path_len:]
            if self.post_path:
                url = '/'.join([url.strip('/'), self.post_path.strip('/')])
            alternates.append({
                'lang': item.Language,
                'url': url.strip('/'),
            })

        return alternates

    def update(self):
        super(CustomAlternateLanguagesViewlet, self).update()
        if parse_version(pam_version) < parse_version('2.0'):
            self.alternates = self.get_alternate_languages_pam_1_x()
        else:
            self.alternates = self.get_alternate_languages_pam_2_x()



# -*- coding: utf-8 -*-
"""Test viewlet of collective.pamfixes."""

# python imports
try:
    import unittest2 as unittest
except ImportError:
    import unittest

# zope imports
try:
    from plone.app.multilingual.api import translate
except ImportError:
    from plone.app.multilingual.tests.utils import makeTranslation as translate

from plone.app.multilingual.browser.viewlets import AlternateLanguagesViewlet

# local imports
from collective.pamfixes import testing
from collective.pamfixes.viewlets import CustomAlternateLanguagesViewlet


class TestViewlet(unittest.TestCase):
    """Validate setup process for collective.pamfixes."""

    layer = testing.COLLECTIVE_PAMFIXES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.app = self.layer['app']

    def test_viewlet_render(self):
        """Validate that the alternate languages viewlet renders correctly."""
        try:
            from plone.app.multilingual.setuphandlers import (
                enable_translatable_behavior,
            )
        except ImportError:
            pass
        else:
            enable_translatable_behavior(self.portal)

        sample_en = self.portal['en']['sample-folder']
        sample_de = translate(sample_en, 'de')
        viewlet_orig = AlternateLanguagesViewlet(
            sample_de,
            self.app.REQUEST,
            None,
            None,
        )
        viewlet_orig.update()

        viewlet = CustomAlternateLanguagesViewlet(
            sample_de,
            self.app.REQUEST,
            None,
            None,
        )
        viewlet.update()
        self.assertTrue(len(viewlet.alternates), 3)



# -*- coding: utf-8 -*-
"""collective.pamfixes interfaces."""

# python imports
from pkg_resources import (
    get_distribution,
    parse_version,
)


pam_distribution = get_distribution('plone.app.multilingual')
pam_version = pam_distribution.version

# version-dependent import
if parse_version(pam_version) < parse_version('2.0'):
    from plone.multilingual.interfaces import (
        ITranslatable,
        ITranslationManager,
    )
else:
    from plone.app.multilingual.interfaces import (
        ITranslatable,
        ITranslationManager,
    )

assert(ITranslatable)
assert(ITranslationManager)



# -*- coding: utf-8 -*-
"""Init and utils."""

import logging

logger = logging.getLogger('collective.pamfixes')

logger.warning(
    'Patching alternate languages viewlet for plone.app.multilingual.'
)



# -*- coding: utf-8 -*-
"""Test setup for collective.pamfixes"""

# zope imports
from OFS.Folder import Folder
from Testing import ZopeTestCase as ztc
from plone.app.testing import (
    applyProfile,
    FunctionalTesting,
    IntegrationTesting,
    PLONE_FIXTURE,
    PloneSandboxLayer,
    setRoles,
    TEST_USER_ID,
)
import transaction

# local imports
import collective.pamfixes


class CollectivePamfixesLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    class Session(dict):
        def set(self, key, value):
            self[key] = value

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        self.loadZCML(package=collective.pamfixes)
        self.loadZCML(name='testing.zcml', package=collective.pamfixes)
        self.loadZCML(name='overrides.zcml', package=collective.pamfixes)

        # Support sessionstorage in tests
        app.REQUEST['SESSION'] = self.Session()
        if not getattr(app, 'temp_folder', None):
            tf = Folder('temp_folder')
            app._setObject('temp_folder', tf)
            transaction.commit()

        ztc.utils.setupCoreSessions(app)

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'collective.pamfixes:testfixture')

        # Empower test user
        setRoles(portal, TEST_USER_ID, ['Manager'])


COLLECTIVE_PAMFIXES_FIXTURE = CollectivePamfixesLayer()


COLLECTIVE_PAMFIXES_INTEGRATION_TESTING = IntegrationTesting(
    bases=(COLLECTIVE_PAMFIXES_FIXTURE,),
    name='CollectivePamfixesLayer:IntegrationTesting'
)


COLLECTIVE_PAMFIXES_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(COLLECTIVE_PAMFIXES_FIXTURE,),
    name='CollectivePamfixesLayer:FunctionalTesting'
)
